#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>   /* EXIT_FAILURE,exit */
#include "config.h"

int digitstr(char *str);
int count_freq(FILE *fp);
int sturges(double count);
double check_precision(char *str);
int main(int argc, char **argv);
double cvar(int n, double s1, double s2);
double kurt(int n, double s2, double s4);
double sdev(int n, double s2);
double serr(int n, double s2);
double skew(int n, double s2, double s3);
double rlo(int n, double s1, double s2, double k);
double rup(int n, double s1, double s2, double k);
double tvar(int n, double s1, double s2);

int screen_w   = 60; /* screen width default */
int only_num   = 0;
int with_stat  = 0;
int out_format = 0;
int print_rank = 0;
int err_f      = 0;
int int_f      = 0;
int st2        = 0;
double prec_level = 1;

int count_freq(FILE *fp)
{
  char buf[BUF];
  char p[BUF];
  int n = 0;
  int i =0 ,k = 0;
  int rank = 0;
  int out_type = 0;
  double x, max, min, rmax, s1, s2, s3, s4, delta;
  double prec = 1;
  ST_RANGE *r;
	
  FILE *fs = tmpfile();   
  if (fs == NULL) 
    exit(EXIT_FAILURE);

  /* read from file */
  while (fgets(buf, BUF,fp) != NULL) {
    if (buf[0] != '#' && (out_type = digitstr(buf))) {
      out_format = MAX(out_format,out_type);
      
      /* ¬ */
      prec = check_precision(buf);
      prec_level = MIN(prec_level,prec);
      x = atof(buf);
      fprintf(fs,"%f\n",x);
      if (n == 0)
        min = max = x;	          /* init variable min and max */
      max = MAX(max,x);
      min = MIN(min,x);
      n++;                        /* N */
      x -= s1;			  /* gap with temp average */
      s1 += x / n;		  /* average */
      s2 += (n - 1) * x * x / n;  /* square  */
    } else
      if (!err_f) {
        for (i = 0; buf[i] != '\0' && buf[i] != '\n' ;i++)
          p[i] = buf[i];
        k++;
        fprintf(stderr,"%3d: error? %s in line %d\n", k, p, n);
      }
  }

  /* num of rank defined by Sturges' rule: k */
  if (st2) {
  /*  rank = sturges_two(n);    */
    rank = sturges(n);
  } else {
    rank = sturges(n);
  }

  /* calc rank width: delta */
  if (int_f) {
    delta = (int)((max - min) / rank);
    rank = (int)((max - min) / delta) + 1;
  } else
    delta = (max - min) / rank;

  rewind(fs); 
	
  /* get memory for rank distribution */
  if ((r = (ST_RANGE *)malloc((rank + 1) * sizeof(ST_RANGE))) == NULL)
    exit(EXIT_FAILURE); 
  
  /* initialize */
  for (i = 0; i < rank ; i++) {
    r[i].cnt = 0;
    if (int_f)
      r[i].range = (int)(min + i * delta);
    else
      r[i].range = min + i * delta;
  }
  
  /* read from tmpfile for rank distribution */
  while (fgets(buf, BUF,fs) != NULL) {
    x = atof(buf);
    s3 += (x-s1) * (x-s1) * (x-s1); 		        /* ٷ׻ */
    s4 += (x-s1) * (x-s1) * (x-s1) * (x-s1); 		/* ٷ׻ */
    i = (int)((x - min) / delta);               /* ؤդ櫓 */
    if (i == rank) /* 곬Ȱפˤ,ʲκ糬 */
      i--;
    r[i].cnt++;
    rmax = MAX(rmax, r[i].cnt);
  }
  fclose(fs); /* end of tmpfile */
	
  if (with_stat) {
    printf("N=%d ", n);
    printf("min=%.3f ", min);
    printf("mean=%.3f ", s1);
    printf("max=%.3f ", max);
    printf("SD=%.3f ", sdev(n,s2)) ;
    printf("SE=%.3f ", serr(n,s2));
    printf("CV=%.3f%% ", cvar(n,s1,s2));
    printf("\n");
    printf("Reliable interval low - upper = %0.3f - %0.3f (95%%)\n",
           rlo(n,s1,s2,2.6), rup(n,s1,s2,2.6));
    printf("skew=%.3f ", skew(n,s2,s3));
    printf("kurtosis=%.3f ", kurt(n,s2,s4));
    printf("rank=%d ", rank);
    printf("delta=%.3f ", delta);
    printf("accuracy=%f\n", prec_level);
  }

  /* output */
  for (i = 0; i < rank; i++) {
    if (print_rank)
      printf("%3d ", i + 1);
    
    if (out_format > 1 || !int_f) 
      printf("%8.2f ", r[i].range);
    else
      printf("%5d ", (int)r[i].range); 
    
    if (!only_num) {
      printf("- :");
      for (k = 0; k < r[i].cnt / (rmax / screen_w) ; k++)
        putchar('*');
    }
    printf(" %d\n", r[i].cnt);
  }
  
  free(r);
  return 0;
}

int main (int argc, char **argv)
{
  FILE *fp;
  int i = 0;
  char *op;

  for (argv++; *argv && argv[0][0] == '-' && argv[0][1]; argv++) {
    for (op = &argv[0][1]; *op; op++) {
      switch (*op) {
      case 'c':
        only_num = 1;
        break;
      case '2':
        st2 = 1;
        break;
      case 'e':
        err_f = 1;
        break;
      case 'i':
        int_f = 1;
        break;
      case 's':
        with_stat = 1;
        break;
      case 'r':
        print_rank = 1;
        break;
      case 'v':
        fprintf(stderr,"%s\n",VERSION);
        exit(0);
      case 'h':
        fprintf(stderr,"%s\n",USAGE);
        exit(0);
      case 'W':
        if (!digitstr(op + 1)) { 
          fprintf(stderr,"%s\n",USAGE);
          exit(0);
        }
        screen_w = atoi(op + 1);
        /*      printf("num of screen width === %d\n",screen_w); */
        while(*(++op + 1))
          ;
        break;
      default:
        fprintf(stderr,"%% %s: invalid option -%c\n", PROG_NAME, *op);
        fprintf(stderr,"%s\n",USAGE);
        exit(0);
      }
    }
  }

  if (*argv == NULL)
    count_freq(stdin);
  else {
    for (; *argv; argv++) {
      if ((fp = fopen(*argv,"r"))==NULL) {
        fprintf(stderr,"can't open %s!\n", *argv);
        exit(1);
      }
      count_freq(fp);
      fclose(fp);
    }
  }
  return 0;
}
